/*
rannu VCL Version 1.04
Producing random-number tables
Copyright 2017 Sogaya

Borland C++Builder 5

*/

#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)
#pragma resource "*.dfm"

#include "rannu.h"
#include "rannuMain.h"
#include "rannuManage.h"
#include "rannuVersion.h"

extern TOKHelpRightDlg *OKHelpRightDlg;
extern TOKRightDlg1 *OKRightDlg1;
MANAGE_MEMO manage_memo; // rannu.h
MANAGE_MEMO *mm = &manage_memo;
TForm1 *Form1;

namespace {
    const int MERSENNE_TWISTER = 1;
    const int XORSHIFT = 6;
}

bool TForm1::header(std::string & pathName) {
    using namespace std;

    ifstream in( pathName.c_str() );
    if( in.fail() ) {
        error = "e15: ifstream in( " + pathName + CLOSE_BRACKET;
        return false;
    }

    stringstream os;
    char buffer[BUFFER_SIZE];
    for(int i = 0; i < FIRST_PART_OF_FILE; ++i) {
        if( in.getline(buffer, BUFFER_SIZE) ) {
            os << buffer << NEW_LINE;
        } else {
            break;
        }

    }

    string s = os.str();

    const AnsiString MEMO_TEXT( s.c_str() );
    Memo1->Text = MEMO_TEXT;
    string PATH_NAME( pathName.c_str() );
    statusBarMessage(PATH_NAME);
    return true;
}

bool TForm1::readManageText() {
    using namespace std;

    char c[BUFFER_SIZE];
    string *p = new string("");
    ifstream in( FULL_PATH_NAME.c_str() );
    if( in.fail() ) {
        error = "e11: ifstream in( " + FULL_PATH_NAME + CLOSE_BRACKET;
        createErrorText(error);
        return false;
    }

    m[0] = FULL_PATH_NAME;
    for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1) {
        if( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            m[line] = *p;
        } else {
            break;
        }

    }

    in.close();
    stringArrayToValues();
    delete p;
    return true;
}

bool TForm1::textHandler() {
    bool result = false;

    checkManageMemory();

// Producing a random number table
    RandomNumber *rn = new RandomNumber(kilo, seed, from, interval_0, interval_1, lineCount, prng, csv, grouping, produce_rannu_text);
    if( rn->createRannuText(pathName) ) {
        result = true;
    } else {
        error = "e13: rn->createRannuText( " +  pathName + CLOSE_BRACKET;
        createErrorText(error);
    }

    delete rn;
    header(pathName);
    return result;
}

bool TForm1::where() {
    using namespace std;

    const int ADJUSTER = 73;
    const int BORDER = 5;
    const string INSIDE("inside");
    const string OUTSIDE("outside");
    POINT pos;

    ::GetCursorPos(&pos);

    bool inside = false;
    int upper_left_x = Left + BORDER;
    int upper_left_y = Top + ADJUSTER;
    int upper_right_x = upper_left_x + ClientWidth;
    int lower_left_y = upper_left_y + ClientHeight - BUTTON_HEIGHT - StatusBar1->Height;
    string here = OUTSIDE;
    if(upper_left_x < pos.x && pos.x < upper_right_x) {
        if(upper_left_y < pos.y && pos.y < lower_left_y) {
            here = INSIDE;
            inside = true;
        }

    }

    stringstream os;
    os << pos.x << " : " << pos.y << " ( " << here << " ) ";
    string s = os.str();
    const AnsiString XY_OUTSIDE( s.c_str() );
    StatusBar1->Panels->Items[1]->Text = XY_OUTSIDE;
    return inside;
}

void TForm1::centering() {
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 0;

    const int SCREEN_WIDTH = Screen->Width;
    const int SCREEN_HEIGHT = Screen->Height;

    Left = SCREEN_WIDTH/2 - Width/2;
    Top = SCREEN_HEIGHT/2 - Height/2;

    standard_main_x = Left;
    standard_main_y = Top;
    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void TForm1::checkManageMemory() {
    if(mm->pleaseCheck) {
        csv = mm->csv;
        from = mm->from;
        grouping = mm->grouping;
        interval_0 = mm->interval_0;
        interval_1 = mm->interval_1;
        kilo = mm->kilo;
        lineCount = mm->lineCount;
        rannu_h = mm->rannu_h;
        rannu_w = mm->rannu_w;
        rannu_x = mm->rannu_x;
        rannu_y = mm->rannu_y;
        pathName = mm->pathName;
        prng = mm->prng;
        produce_rannu_text = mm->produce_rannu_text;
        seed = mm->seed;

        mm->pleaseCheck = false;
    }

}

void TForm1::cou99(bool arithmetic) {
    using namespace std;

    bool analysis = !arithmetic;
    char file_name[FILE_NAME_SIZE];
    const string COLON_GAP(": ");
    Count99 *c = new Count99();
    double d = (double)(interval_0 + interval_1);
    double m = d/2.0;
    int columnCount, n;
    string floating_point_number, s;

    pathName = mm->pathName;
    strcpy(file_name, pathName.c_str() );
    columnCount = c->countingColumns(file_name);
    s = c->tabulating(m, file_name, columnCount, analysis, arithmetic);

// mean or variance
    n = strcspn( s.c_str(), "0123456789");
    s = s.substr(n);
    n = strspn( s.c_str(), ".0123456789");

    floating_point_number = s.substr(0, n);
    statusBarMessage(floating_point_number);

// table
    n = s.find(COLON_GAP);
    s = s.substr( n - 2 );
    const AnsiString MEMO_TEXT( s.c_str() );
    Memo1->Text = MEMO_TEXT;
    delete c;
}

void TForm1::largeHandler() {
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 1;

    Top = CORNER_X;
    Left = CORNER_Y;
    Width = LARGE_PANE_X;
    Height = LARGE_PANE_Y;

    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void TForm1::lotoHandler(int a) {
    using namespace std;

    string INTERVAL_1_31("interval of 1-31");
    string INTERVAL_1_37("interval of 1-37");
    string INTERVAL_1_40("interval of 1-40");
    string INTERVAL_1_43("interval of 1-43");

    switch(a) {
    case 31:
        statusBarMessage(INTERVAL_1_31);
        break;
    case 37:
        statusBarMessage(INTERVAL_1_37);
        break;
    case 40:
        statusBarMessage(INTERVAL_1_40);
        break;
    case 43:
        statusBarMessage(INTERVAL_1_43);
        break;
    default:
        a = 37;
        statusBarMessage(INTERVAL_1_37);
        break;
    }

    grouping = false;
    interval_0 = 1;
    interval_1 = a;
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
}

void TForm1::selectingPRNG(int r) {
    const int MERSENNE_TWISTER = 1;
    const int XORSHIFT = 6;

    switch(r) {
    case XORSHIFT:
        Edit1->Text = "xorshift";
        prng = XORSHIFT;
        break;
    default:
        rannu_w = Width;
        if(rannu_w > SMALL_PANE_WIDTH) {
            Edit1->Text = "Mersenne Twister (MT)";
        } else {
            Edit1->Text = "MT";
        }

        prng = MERSENNE_TWISTER;
        break;
    }

    mm->prng = prng;
}

void TForm1::standardHeader(bool perfect) {
    WindowState = wsNormal;
    checkManageMemory();

    if(perfect) {
        whichCorner = 0;
        mm->whichCorner = 0;
        Left = standard_main_x;
        Top = standard_main_y;
    }

    Height = MAIN_HEIGHT;
    Width = MAIN_WIDTH;

// Moving the Rannu window
    const int SCREEN_WIDTH = Screen->Width;
    const int SCREEN_HEIGHT = Screen->Height;

    switch(whichCorner) {
    case 1:
        Left = 0;
        Top = 0;
        break;
    case 2:
        Left = SCREEN_WIDTH - Width;
        Top = 0;
        break;
    case 3:
        Left = SCREEN_WIDTH - Width;
        Top = SCREEN_HEIGHT - Height - TASK_BAR_HEIGHT;
        break;
    case 4:
        Left = 0;
        Top = SCREEN_HEIGHT - Height - TASK_BAR_HEIGHT;
        break;
    default:
        Left = SCREEN_WIDTH/2 - Width/2;
        Top = SCREEN_HEIGHT/2 - Height/2;
        break;
    }

    if(perfect) {
        standard_main_x = Left;
        standard_main_y = Top;
    }

    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void TForm1::statusBarMessage(std::string & s) {
    statusBarText = s;
    const AnsiString MESSAGES( s.c_str() );
    StatusBar1->Panels->Items[0]->Text = MESSAGES;
}

void TForm1::stringArrayToValues() {
    char *stop;

    csv = (ZERO == m[1]) ? false : true;
    pathName = m[2] + RANNU_TEXT;
    kilo = strtol( m[3].c_str(), &stop, 10);
    interval_0 = stringToInteger(m[4]);
    interval_1 = stringToInteger(m[5]);
    grouping = (ZERO == m[6]) ? false : true;
    from = stringToInteger(m[7]);
    seed = strtol( m[8].c_str(), &stop, 10);
    lineCount = stringToInteger(m[9]);

    mm->csv = csv;
    mm->from = from;
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
    mm->kilo = kilo;
    mm->lineCount = lineCount;
    mm->pathName = pathName;
    mm->pleaseCheck = true;
    mm->seed = seed;
}

void TForm1::tracking() {
    const int ADJUSTER = 1;
    const int CLIENT_WIDTH = ClientWidth - ADJUSTER;
    const int MID = ClientWidth/2;

// MT or xorshift
    Edit1->Width = MID;

// Count button
    Button1->Left = MID;

// Large button
    Button2->Left = CLIENT_WIDTH - BUTTON_WIDTH;
}

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner), ARITHMETIC_MEAN(true), CLOSE_BRACKET(" )"), IMPERFECT(false), MA(MANAGE_TEXT), NEW_LINE("\x0D\x0A"), PERFECT(true), RANNU_TEXT("rannu.txt"), VARIANCE(false), ZERO("0")
{
    const int CLIENT_WIDTH = 511;
    const int MID = 260;

// MT or xorshift
    Edit1->Left = 0;
    Edit1->Top = 0;
    Edit1->Width = MID;
    Edit1->Height = BUTTON_HEIGHT;

// Count button
    Button1->Left = MID;
    Button1->Top = 0;
    Button1->Width = BUTTON_WIDTH;
    Button1->Height = BUTTON_HEIGHT;

// Large button
    Button2->Left = CLIENT_WIDTH - BUTTON_WIDTH;
    Button2->Top = 0;
    Button2->Width = BUTTON_WIDTH;
    Button2->Height = BUTTON_HEIGHT;

// text area
    Memo1->Align = alClient;
    Memo1->Text = "";

// status bar
    statusBarText = mm->FULL_PATH_NAME;
    mm->statusBarText = statusBarText;
    const AnsiString MESSAGES( statusBarText.c_str() );
    StatusBar1->Panels->Items[0]->Text = MESSAGES;

    checkManageMemory();
    FULL_PATH_NAME = mm->FULL_PATH_NAME;
    prng = mm->prng;
    produce_rannu_text = mm->produce_rannu_text;
    showWhere = mm->showWhere;
    selectingPRNG(prng);

    standardHeader(IMPERFECT);
    manage_text = MA; // AnsiString
    untouched = FileAge(manage_text);
    firstSession = true;
    Timer1->Interval = NON_SAMPLING_PERIOD;
}

void __fastcall TForm1::Bingo51Click(TObject *Sender)
{
    lotoHandler(40);
}

void __fastcall TForm1::Button1Click(TObject *Sender)
{
    cou99(VARIANCE);
}

void __fastcall TForm1::Button1KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    char buffer[BUFFER_SIZE];

    if( Shift.Contains(ssCtrl) ) {
    	switch(Key) {
        case '0':
            strcpy(buffer, statusBarText.c_str() );
            Clipboard()->SetTextBuf(buffer);
            break;
        default:
            return;
        }

    }

}

void __fastcall TForm1::Button2Click(TObject *Sender)
{
    largeHandler();
}

void __fastcall TForm1::Button2KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    char buffer[BUFFER_SIZE];

    if( Shift.Contains(ssCtrl) ) {
    	switch(Key) {
        case '0':
            strcpy(buffer, statusBarText.c_str() );
            Clipboard()->SetTextBuf(buffer);
            break;
        default:
            return;
        }

    }

}

void __fastcall TForm1::Center1Click(TObject *Sender)
{
    centering();
}

void __fastcall TForm1::Copy1Click(TObject *Sender)
{
    char buffer[BUFFER_SIZE];
    strcpy(buffer, statusBarText.c_str() );
    Clipboard()->SetTextBuf(buffer);
}

void __fastcall TForm1::Corner1Click(TObject *Sender)
{
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 1;

    Top = 0;
    Left = 0;
    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void __fastcall TForm1::Count1Click(TObject *Sender)
{
    cou99(VARIANCE);
}

void __fastcall TForm1::CSV1Click(TObject *Sender)
{
    mm->csv = true;
    mm->pleaseCheck = true;
    textHandler();
}

void __fastcall TForm1::Diagonal1Click(TObject *Sender)
{
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 3;

    const int SCREEN_WIDTH = Screen->Width;
    const int SCREEN_HEIGHT = Screen->Height;

    Left = SCREEN_WIDTH - Width;
    Top = SCREEN_HEIGHT - Height - TASK_BAR_HEIGHT;
    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void __fastcall TForm1::Edit1KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    char buffer[BUFFER_SIZE];

    if( Shift.Contains(ssCtrl) ) {
    	switch(Key) {
        case '0':
            strcpy(buffer, statusBarText.c_str() );
            Clipboard()->SetTextBuf(buffer);
            break;
        default:
            return;
        }

    }

}

void __fastcall TForm1::Exit1Click(TObject *Sender)
{
    Close();
}

void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    char buffer[BUFFER_SIZE];

    if( Shift.Contains(ssCtrl) ) {
    	switch(Key) {
        case '0':
            strcpy(buffer, statusBarText.c_str() );
            Clipboard()->SetTextBuf(buffer);
            break;
        default:
            return;
        }

    }

}

void __fastcall TForm1::FormResize(TObject *Sender)
{
    tracking();
}

void __fastcall TForm1::Grouping1Click(TObject *Sender)
{
    std::string INTERVAL_0_63("interval of 0-63");

    grouping = true;
    interval_0 = 0;
    interval_1 = 63;

    statusBarMessage(INTERVAL_0_63);
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
}

void __fastcall TForm1::Large1Click(TObject *Sender)
{
    largeHandler();
}

void __fastcall TForm1::Loto61Click(TObject *Sender)
{
    lotoHandler(43);

}

void __fastcall TForm1::Loto71Click(TObject *Sender)
{
    lotoHandler(37);

}

void __fastcall TForm1::Manage1Click(TObject *Sender)
{
    OKRightDlg1->setting();
    OKRightDlg1->ShowModal();
}

void __fastcall TForm1::Mean1Click(TObject *Sender)
{
    cou99(ARITHMETIC_MEAN);    
}

void __fastcall TForm1::Memo1KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    char buffer[BUFFER_SIZE];

    if( Shift.Contains(ssCtrl) ) {
    	switch(Key) {
        case '0':
            strcpy(buffer, statusBarText.c_str() );
            Clipboard()->SetTextBuf(buffer);
            break;
        default:
            return;
        }

    }

}

void __fastcall TForm1::Miniloto1Click(TObject *Sender)
{
    lotoHandler(31);

}

void __fastcall TForm1::MT1Click(TObject *Sender)
{
    selectingPRNG(MERSENNE_TWISTER);
}

void __fastcall TForm1::Right1Click(TObject *Sender)
{
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 2;

    const int SCREEN_WIDTH = Screen->Width;
    Left = SCREEN_WIDTH - Width;
    Top = 0;
    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void __fastcall TForm1::Small1Click(TObject *Sender)
{
    WindowState = wsNormal;

    Height = SMALL_PANE_HEIGHT;
    Width = SMALL_PANE_WIDTH;

    if(prng == MERSENNE_TWISTER) {
        Edit1->Text = "MT";
    }

    mm->rannu_h = Height;
    mm->rannu_w = Width;
    mm->pleaseCheck = true;
    centering();
}

void __fastcall TForm1::Standard1Click(TObject *Sender)
{
    standardHeader(PERFECT);
}

void __fastcall TForm1::Text1Click(TObject *Sender)
{
    mm->csv = false;
    mm->pleaseCheck = true;
    textHandler();
}

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    checkManageMemory();

    if(showWhere) {
// Showing where the cursor is
        where();
    }

// Checking if MANAGE_TEXT has been modified
    int m = FileAge(manage_text);
    if(m != untouched) {
        untouched = m;
        readManageText();
        statusBarMessage(FULL_PATH_NAME);
        OKRightDlg1->setting();
    }

    if(firstSession) {
        Memo1->SetFocus();
        firstSession = false;
    }

}

void __fastcall TForm1::Under1Click(TObject *Sender)
{
    WindowState = wsNormal;
    checkManageMemory();
    whichCorner = 4;

    const int SCREEN_HEIGHT = Screen->Height;
    Left = 0;
    Top = SCREEN_HEIGHT - Height - TASK_BAR_HEIGHT;
    mm->rannu_x = Left;
    mm->rannu_y = Top;
    mm->rannu_w = Width;
    mm->rannu_h = Height;
    mm->whichCorner = whichCorner;
    mm->pleaseCheck = true;
}

void __fastcall TForm1::Version1Click(TObject *Sender)
{
    OKHelpRightDlg->ShowModal();
}

void __fastcall TForm1::xorshift1Click(TObject *Sender)
{
    selectingPRNG(XORSHIFT);
}
